/* The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Eric Hodel's <drbrain@segment7.net> code.
 * 
 * The Initial Developer of the Original Code is Eric Hodel.
 * Portions created by Eric Hodel are Copyright (C) 2001 Eric
 * Hodel. All Rights Reserved.
 * Other Contributors:
 * Christopher Hoess <choess@force.stwing.upenn.edu>
 */

const nsIHttpChannel = Components.interfaces.nsIHttpChannel;
const nsIIOService = Components.interfaces.nsIIOService;
const NS_IOSERVICE_CID_STR = "{9ac9e770-18bc-11d3-9337-00104ba0fd40}";
var ioService = null;
var curDocHTTPHeaders = null;

/*
 * get HTTP headers
 */
function getHTTPHeaders(url)
{
  if (!ioService)
  {
    ioService = Components.classesByID[NS_IOSERVICE_CID_STR].getService();
    ioService = ioService.QueryInterface(nsIIOService);
  }
  doGetHTTPHeaders(url);
}

function doGetHTTPHeaders(url)
{
  ddump("doGetHTTPHeaders: url = " + url);
  //if (!channel) return (false);

  //var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  //var ioService = getService("@mozilla.org/network/io-service;1", "nsIIOService");
  var ioService = null;
  var httpChannel = null;

  try
  {
    ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces["nsIIOService"]);
    ddump("doGetHTTPHeaders: ioService = " + ioService);
    httpChannel = ioService.newChannel(url, null);
    ddump("doGetHTTPHeaders: httpChannel = " + httpChannel);
    ddump("doGetHTTPHeaders: Components.interfaces.nsIHttpChannel = " + Components.interfaces.nsIHttpChannel);
  httpChannel = httpChannel.QueryInterface(Components.interfaces.nsIHttpChannel);
    ddump("doGetHTTPHeaders: httpChannel = " + httpChannel);

    if (!httpChannel) return;

    ddump("doGetHTTPHeaders: httpChannel.requestMethod = " + requestMethod );

  httpChannel.visitResponseHeaders(new HttpHeaderVisitor());
  return;



    curDocHTTPHeaders = new Array();

  ddump("doGetHTTPHeaders: responseHeader = " + httpChannel.getResponseHeader("Link"));
  return;
    var headerEnum = httpChannel.getResponseHeaderEnumerator();
    if (!headerEnum) return;

    var httpResponse;

    while (headerEnum.hasMoreElements())
    {
      httpResponse = headerEnum.getNext();
      httpResponse = httpResponse.QueryInterface(Components.interfaces.nsIHTTPHeader);
      if (httpResponse)
      {
        httpField = httpResponse.GetFieldName();
        if (httpField.toLowerCase() == "link")
        {
          httpValue = httpResponse.GetValue();
          curDocHTTPHeaders.push(httpValue);
        }
      }
    }
  }
  catch (e)
  {
    ddump("doGetHTTPHeaders() caught error " + e + "... returning");
    return;
  }
}

function HttpHeaderVisitor() { }
HttpHeaderVisitor.prototype.visitHeader = function(header, value) {
  dump(header + ": " + value + "\n");
}


/*
 * Parse Link HTTP headers per RFC 2068
 */
function parseLinkHeader(content)
{
  const LS_NEWLINK = 0;
  const LS_HREFSTART = 1;
  const LS_HREFEND = 2;
  const LS_NEWPARAM = 3;
  const LS_PARAMSTART = 4;
  const LS_PARAMEND = 5;
  const LS_RELPARAMVALSTART = 6;
  const LS_PARAMVALSTART = 7;
  const LS_PARAMVALEND = 8;
  const LS_PARAMVALESC = 9;

  var link;
  var newparam;
  var newparamval;

  contentlen = content.length;
  state = LS_NEWLINK;
  for (var i = 0; i < contentlen; i++)
  {
    curchar = content.charAt(i);
    if (state != LS_RELPARAMVALSTART && curchar == " ") continue;
    switch (state)
    {
      case LS_NEWLINK:
        if (curchar == "<")
        {
          state = LS_HREFSTART;
          starthref = i;
          link = window._content.document.createElement("link");
        }
        break;
      case LS_HREFSTART:
        if (curchar == ">")
        {
          state = LS_HREFEND;
          endhref = i;
          if (endhref-starthref > 1)
          {
            link.setAttribute("href", content.slice(starthref+1, endhref));
          }
          else
          {
            link.setAttribute("href", "");
          }
        }
        break;
      case LS_HREFEND:
        if (curchar == ";")
        {
          state = LS_NEWPARAM;
        } else if (curchar == ",")
        {
          state = LS_NEWLINK;
        }
        break;
      case LS_NEWPARAM:
        switch (curchar)
        {
          case "=":
            state = LS_PARAMEND;
          case ";":
            break;
          default:
            state = LS_PARAMSTART;
            startparam = i;
            break;
        }
        break;
      case LS_PARAMSTART:
        if (curchar == "=")
        {
          state = LS_PARAMEND;
          endparam = i;
          newparam = content.slice(startparam, endparam);
        }
        break;
      case LS_PARAMEND:
        if (curchar == '"')
        {
          state = LS_PARAMVALSTART;
          startparamval = i;
        }
        else {
          switch (newparam.toLowerCase())
          {
            case "rel":
            case "rev":
              state = LS_RELPARAMVALSTART;
              startparamval = i;
              break;
          }
        }
        break;
      case LS_RELPARAMVALSTART:
        switch (curchar)
        {
          case " ":
          case ";":
          case ",":
            endparamval = i;
            if (endparamval-startparamval > 0)
            {
              link.setAttribute(newparam, content.slice(startparamval, endparamval) );
            }
            switch (curchar)
            {
              case " ":
                state = LS_PARAMVALEND;
              case ";":
                state = LS_NEWPARAM;
                startparam = i;
                break;
              case ",":
                state = LS_NEWLINK;
                break;
            }
        }
        break;
      case LS_PARAMVALSTART:
        if (curchar == '"')
        {
          state = LS_PARAMVALEND;
          endparamval = i;
          if (endparamval-startparamval > 1)
          {
            newparamval = content.slice(startparamval+1, endparamval);
            newparamval = newparamval.replace(/\\(.)/g, "$1");
            link.setAttribute(newparam, newparamval );
          }
          else
          {
            link.setAttribute(content.slice(startparam, endparam), "");
          }
        } else if (curchar == "\\") state = LS_PARAMVALESC;
        break;
      case LS_PARAMVALESC:
        state = LS_PARAMVALSTART;
        break;
      case LS_PARAMVALEND:
        switch (curchar)
        {
          case ";":
            state = LS_NEWPARAM;
            startparam = i;
            break;
          case ",":
            state = LS_NEWLINK;
            break;
        }
        break;
    }
  }

  // XXX: cleanup
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces["nsIIOService"]);
  var base = ioService.newURI(window._content.document.base 
      ? window._content.document.base
      : window._content.document.URL, null);
  link.href = base.resolve(link.href);

  return link;
}
